//
// This file is part of the GNU ARM Eclipse distribution.
// Copyright (c) 2014 Liviu Ionescu.
//

#ifndef SERVO_H_
#define SERVO_H_

#include "stm32f30x.h"

// ----- LED definitions ------------------------------------------------------

// Adjust these definitions for your own board.

// STM32F3DISCOVERY definitions (GREEN led, E15, active high)
// (SEGGER J-Link device name: STM32F303VC).

// I pin sono PD12, PD13 e PD14
// Port numbers: 0=A, 1=B, 2=C, 3=D, 4=E, 5=F, 6=G, ...
#define SERVO_PORT_NUMBER               (3)
#define SERVO_PIN_NUMBER                (12)



#define SERVO_GPIOx(_N)                 ((GPIO_TypeDef *)(GPIOA_BASE + (GPIOB_BASE-GPIOA_BASE)*(_N)))
#define SERVO_PIN_MASK(_N)              (1 << (_N))
#define SERVO_RCC_MASKx(_N)             (RCC_AHBPeriph_GPIOA << (_N))


#define TIMER	TIM4
#define TIMER_CHANNEL_INIT	TIM_OC1Init
#define TIMER_CHANNEL_PRELOAD_CONFIG TIM_OC1PreloadConfig
#define TIMER_CHANNEL_SET_COMPARE	TIM_SetCompare1




#define TIMER_AF	GPIO_AF_2

#define TIMER_RCC_BIT RCC_APB1Periph_TIM4

/* The following times are in milliseconds */
#define SERVO_ANGLE_SPAN 180
#define SERVO_PERIOD 20
#define SERVO_MIN_PULSE 0.6
#define SERVO_MAX_PULSE 2.4
//#define SERVO_MIN_PULSE 0.55
//#define SERVO_MAX_PULSE 2.5

#define TIMER_CLOCK_FREQUENCY ((unsigned)(SERVO_ANGLE_SPAN*1000/(SERVO_MAX_PULSE-SERVO_MIN_PULSE)))
#define TIMER_PERIOD ((unsigned)(TIMER_CLOCK_FREQUENCY*SERVO_PERIOD/1000))
#define TIMER_OFFSET ((unsigned)(TIMER_CLOCK_FREQUENCY*(SERVO_MIN_PULSE+SERVO_MAX_PULSE)/2000))






// ----------------------------------------------------------------------------

extern
void
servo_init(void);

extern
void
servo_set_angle(short angle);

// ----------------------------------------------------------------------------



// ----------------------------------------------------------------------------

#endif // SERVO_H_
