#include "stm32f30x.h"
#include "Button.h"
#include "Timer.h"



#define BUTTON_PORT GPIOA
#define BUTTON_PIN 0
#define BUTTON_PIN_MASK (1<<BUTTON_PIN)
#define BUTTON_RCC_MASK RCC_AHBPeriph_GPIOA

#define STABILIZATION_TIME 3
#define STABILIZATION_DELAY 1

void button_init(void) {
	  // Enable GPIO Peripheral clock
	  RCC_AHBPeriphClockCmd(BUTTON_RCC_MASK, ENABLE);

	  GPIO_InitTypeDef GPIO_InitStructure;

	  // Configure pin in input mode
	  GPIO_InitStructure.GPIO_Pin = BUTTON_PIN_MASK;
	  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
	  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	  GPIO_Init(BUTTON_PORT, &GPIO_InitStructure);
}




int button_read(void) {
	unsigned data=GPIO_ReadInputData(BUTTON_PORT);
	return (data & BUTTON_PIN_MASK)!=0;
}

void button_init_debouncing_state(ButtonDebouncingState *s) {
	s->prev=0;
	s->stable=0;
	s->duration=0;
}

int button_read_debounced(ButtonDebouncingState *s) {
	int button=button_read();
	if (button == s->prev) {
		s->duration++;
	} else {
		s->duration=0;
		s->prev=button;
	}
	if (s->duration>=STABILIZATION_TIME) {
		s->stable=s->prev;
		s->duration=STABILIZATION_TIME;
	}
	return s->stable;
}
