//
// This file is part of the GNU ARM Eclipse distribution.
// Copyright (c) 2014 Liviu Ionescu.
//

#include "stm32f30x_tim.h"

#include "Servo.h"

// ----------------------------------------------------------------------------

void
servo_init()
{
  // Enable GPIO Peripheral clock
  RCC_AHBPeriphClockCmd(SERVO_RCC_MASKx(SERVO_PORT_NUMBER), ENABLE);

  GPIO_InitTypeDef GPIO_InitStructure;

  // Configure pin in output push/pull mode
  GPIO_InitStructure.GPIO_Pin = SERVO_PIN_MASK(SERVO_PIN_NUMBER);
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(SERVO_GPIOx(SERVO_PORT_NUMBER), &GPIO_InitStructure);

  GPIO_PinAFConfig(SERVO_GPIOx(SERVO_PORT_NUMBER), SERVO_PIN_NUMBER, TIMER_AF);


  // Enable Timer clock
  RCC_APB1PeriphClockCmd(TIMER_RCC_BIT, ENABLE);

  // Configure timer
  RCC_ClocksTypeDef RCC_Clocks;

  RCC_GetClocksFreq(&RCC_Clocks);
  uint32_t prescaler=RCC_Clocks.PCLK2_Frequency/TIMER_CLOCK_FREQUENCY-1;
  TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure;
  TIM_TimeBaseStructure.TIM_Period = TIMER_PERIOD-1;
  TIM_TimeBaseStructure.TIM_Prescaler = prescaler;
  TIM_TimeBaseStructure.TIM_ClockDivision = TIM_CKD_DIV1;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
  TIM_TimeBaseStructure.TIM_RepetitionCounter = 0;
  TIM_TimeBaseInit(TIMER, &TIM_TimeBaseStructure);

  // Configure timer channel
  TIM_OCInitTypeDef TIM_OCInitStructure;

  TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM1;
  TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
  TIM_OCInitStructure.TIM_Pulse = 0;
  TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;
  TIM_OCInitStructure.TIM_OCIdleState = TIM_OCIdleState_Set;
  // These settings must be applied on the timer 1.
  TIM_OCInitStructure.TIM_OutputNState = TIM_OutputNState_Disable;
  TIM_OCInitStructure.TIM_OCNPolarity = TIM_OCNPolarity_High;
  TIM_OCInitStructure.TIM_OCNIdleState = TIM_OCNIdleState_Set;
  TIMER_CHANNEL_INIT(TIMER, &TIM_OCInitStructure);
  TIMER_CHANNEL_PRELOAD_CONFIG(TIMER, TIM_OCPreload_Enable);



  // Enable timer
  servo_set_angle(90);


  TIM_Cmd(TIMER, ENABLE);
}

// ----------------------------------------------------------------------------

extern
void
servo_set_angle(short angle) {
	TIMER_CHANNEL_SET_COMPARE(TIMER, (uint16_t)(TIMER_OFFSET+angle));
}
