//
// This file is part of the GNU ARM Eclipse distribution.
// Copyright (c) 2014 Liviu Ionescu.
//

#include "StepMotor.h"


/*---------------------------------------------------------------
 * Constants
 */
const static uint8_t sequence[] = { 0x01, 0x03, 0x02, 0x06, 0x04, 0x0C, 0x08, 0x09 };
#define SEQUENCE_SIZE (sizeof(sequence)/sizeof(sequence[0]))


/*-------------------------------------------
 * Static function prototypes
 *
 */

static void step_set(uint8_t mask);

// ----------------------------------------------------------------------------

void step_init(StepMotorState *state)
{
  // Enable GPIO Peripheral clock
  RCC_AHBPeriphClockCmd(STEP_RCC_MASKx(STEP_PORT_NUMBER), ENABLE);

  GPIO_InitTypeDef GPIO_InitStructure;

  // Configure pin in output push/pull mode
  GPIO_InitStructure.GPIO_Pin = STEP_PIN_ALL_MASK;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(STEP_GPIOx(STEP_PORT_NUMBER), &GPIO_InitStructure);

  state->step = 0;
  step_set(sequence[state->step]);
}

void step_forward(StepMotorState *state) {
	state->step ++;
	if (state->step>= SEQUENCE_SIZE)
		state->step=0;
	step_set(sequence[state->step]);
}

void step_backward(StepMotorState *state) {
	state->step --;
	if (state->step<0)
		state->step=SEQUENCE_SIZE-1;
	step_set(sequence[state->step]);
}

void step_stop(StepMotorState *state) {
	step_set(0);
}


// ----------------------------------------------------------------------------

static void step_set(uint8_t mask) {
	unsigned set_mask=mask << STEP_PIN_NUMBER_MIN;
	unsigned reset_mask = ~set_mask & STEP_PIN_ALL_MASK;
	if (reset_mask!=0)
		GPIO_ResetBits(STEP_GPIOx(STEP_PORT_NUMBER), reset_mask);
	if (set_mask!=0)
		GPIO_SetBits(STEP_GPIOx(STEP_PORT_NUMBER), set_mask);
}
