//
// This file is part of the GNU ARM Eclipse distribution.
// Copyright (c) 2014 Liviu Ionescu.
//

// ----------------------------------------------------------------------------

#include <stdio.h>
#include <stdint.h>

#include "diag/Trace.h"

#include "Timer.h"
#include "StepMotor.h"
#include "Button.h"


#define TRUE 1
#define FALSE 0

// ----------------------------------------------------------------------------
//
// Standalone STM32F3 led blink sample (trace via DEBUG).
//
// In debug configurations, demonstrate how to print a greeting message
// on the trace device. In release configurations the message is
// simply discarded.
//
// Then demonstrates how to blink a led with 1Hz, using a
// continuous loop and SysTick delays.
//
// Trace support is enabled by adding the TRACE macro definition.
// By default the trace messages are forwarded to the DEBUG output,
// but can be rerouted to any device or completely suppressed, by
// changing the definitions required in system/src/diag/trace_impl.c
// (currently OS_USE_TRACE_ITM, OS_USE_TRACE_SEMIHOSTING_DEBUG/_STDOUT).
//
// The external clock frequency is specified as a preprocessor definition
// passed to the compiler via a command line option (see the 'C/C++ General' ->
// 'Paths and Symbols' -> the 'Symbols' tab, if you want to change it).
// The value selected during project creation was HSE_VALUE=8000000.
//
// Note: The default clock settings take the user defined HSE_VALUE and try
// to reach the maximum possible system clock. For the default 8MHz input
// the result is guaranteed, but for other values it might not be possible,
// so please adjust the PLL settings in system/src/cmsis/system_stm32f30x.c
//

// ----- Timing definitions -------------------------------------------------


#define LOOP_DELAY 5

typedef enum  {
	DIR_NONE,
	DIR_FORWARD,
	DIR_BACKWARD
} Direction;

// ----- main() ---------------------------------------------------------------

// Sample pragmas to cope with warnings. Please note the related line at
// the end of this function, used to pop the compiler diagnostics status.
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wunused-parameter"
#pragma GCC diagnostic ignored "-Wmissing-declarations"
#pragma GCC diagnostic ignored "-Wreturn-type"

int
main(int argc, char* argv[])
{
  // Send a greeting to the trace device (skipped on Release).
  //trace_puts("Hello ARM World!");

  // At this stage the system clock should have already been configured
  // at high speed.
  //trace_printf("System clock: %uHz\n", SystemCoreClock);

  ButtonDebouncingState debouncing_state;
  StepMotorState motor_state;
  Direction dir=DIR_FORWARD;

  timer_start();

  step_init(&motor_state);
  button_init();
  button_init_debouncing_state(&debouncing_state);
  
  int prev_button=FALSE;

  // Infinite loop
  while (1)
    {

	  int button=button_read_debounced(&debouncing_state);

	  if (button && !prev_button) {
		  switch(dir) {
		  	  case DIR_NONE: dir=DIR_FORWARD; break;
		  	  case DIR_FORWARD: dir=DIR_BACKWARD; break;
		  	  case DIR_BACKWARD: dir=DIR_NONE; break;
		  }

	  }
	  prev_button=button;



	  if (dir==DIR_FORWARD)
		  step_forward(&motor_state);
	  else if (dir==DIR_BACKWARD)
		  step_backward(&motor_state);
	  else
		  step_stop(&motor_state);


	  timer_sleep(LOOP_DELAY);
    }
  // Infinite loop, never return.
}

#pragma GCC diagnostic pop

// ----------------------------------------------------------------------------
